package com.ho.test.reflexion;

import java.lang.reflect.Constructor;
import java.lang.reflect.Field;
import java.lang.reflect.Method;

/**
 * Reflection Tools Class
 * @author Huseyin OZVEREN
 *
 */
public class ReflectionTools {
	public ReflectionTools() { }

	// Classname 
	public String getName(Object o) throws Exception {
		Class c = o.getClass();
		String name = c.getName();
		System.out.println("getName=" + name);
		return name;
	}

	// Field Object corresponding to a specific public field
	public Field getField(Object o, String name) throws Exception {
		Class c = o.getClass();
		Field f = c.getField(name);
		System.out.println("getField=" + f.toString());
		return f;
	}

	// Public fields of an Object
	public Field[] getFields(Object o) throws Exception {
		Class c = o.getClass();
		Field[] ftab = c.getFields();
		for (int i = 0; i < ftab.length; i++) {
			System.out.println("getFields ftab[" + i + "]=" + ftab[i].toString());
		}
		return ftab;
	}

	// Method Object corresponding to a specific public method with the defined parameters by the parameterTypes tables
	public Method getMethod(Object o, String name, Class... parameterTypes)throws Exception {
		Class c = o.getClass();
		Method m = c.getMethod(name, parameterTypes);
		System.out.println("getMethod=" + m.toString());
		return m;
	}

	// Public methods of an Object
	public Method[] getMethods(Object o) throws Exception {
		Class c = o.getClass();
		Method[] mtab = c.getMethods();
		for (int i = 0; i < mtab.length; i++) {
			System.out.println("getMethods mtab[" + i + "]=" + mtab[i].toString());
		}
		return mtab;
	}

	// Constructor object corresponding to a specific public constructor with the defined parameters by the parameterTypes table
	public Constructor getConstructor(Object o, Class... parameterTypes) throws Exception {
		Class c = o.getClass();
		Constructor cons = c.getConstructor(parameterTypes);
		System.out.println("getConstructor=" + cons.toString());
		return cons;
	}

	// Public Constructors of an Object
	public Constructor[] getConstructors(Object o) throws Exception {
		Class c = o.getClass();
		Constructor[] constab = c.getConstructors();
		for (int i = 0; i < constab.length; i++) {
			System.out.println("getConstructors constab[" + i + "]=" + constab[i].toString());
		}
		return constab;
	}

	// Implemented interfaces of an Object
	public Class[] getInterfaces(Object o) throws Exception {
		Class c = o.getClass();
		Class[] itab = c.getInterfaces();
		for (int i = 0; i < itab.length; i++) {
			System.out.println("getInterfaces itab[" + i + "]=" + itab[i].toString());
		}
		return itab;
	}

	// Super classes of an Object
	public Class getSuperclass(Object o) throws Exception {
		Class c = o.getClass();
		Class superclass = c.getSuperclass();
		System.out.println("getSuperclass=" + superclass.toString());
		return superclass;
	}

	// Package object corresponding of an Object
	public Package getPackage(Object o) throws Exception {
		Class c = o.getClass();
		Package pck = c.getPackage();
		System.out.println("getPackage=" + pck.toString());
		return pck;
	}

	// Change the value of a public field
	public void changeFieldValue(Object o, String name, Object val) throws Exception {
		Field f = this.getField(o, name);
		f.set(o,val);
	}
	
	// Return a public field 
	public Object getFieldValue(Object o, String name) throws Exception {
		Field f = this.getField(o, name);
		return f.get(o);
	}

	// Invoke a public method
	public Object invokeMethod(Object o, String name, Object[] args) throws Exception {
	  Class[] paramTypes = null;
	  if(args != null)
	  {
	    paramTypes = new Class[args.length];
	    for(int i=0;i<args.length;++i){
	      paramTypes[i] = args[i].getClass();
	    }
	  }
	  Method m = this.getMethod(o, name, paramTypes);
	  return m.invoke(o,args);
	}
	
	//######################################################################################################
	// Important Note: unlike "getFields" and "getMethods" methods, the methods "getDeclaredFields" and "getDeclaredMethods"
	// do not return the information inherited. In this case, it is necessary to go ask the parent class.
	//######################################################################################################

	// Get a specific Field object corresponding to public, private or protected field
	public Field getDeclaredField(Object o, String name) throws Exception {
		Class c = o.getClass();
		Field f = c.getDeclaredField(name);
		f.setAccessible(true);
		System.out.println("getDeclaredField=" + f.toString());
		return f;
	}

	// Public, private and protected fields of an Object
	public Field[] getDeclaredFields(Object o) throws Exception {
		Class c = o.getClass();
		Field[] ftab = c.getDeclaredFields();
		for (int i = 0; i < ftab.length; i++) {
			ftab[i].setAccessible(true);
			System.out.println("getDeclaredFields ftab[" + i + "]=" + ftab[i].toString());
		}
		return ftab;
	}

	// Method object corresponding to a specific public, private or protected method with the defined parameters by the parameterTypes table
	public Method getDeclaredMethod(Object o, String name, Class... parameterTypes)throws Exception {
		Class c = o.getClass();
		Method m = c.getDeclaredMethod(name, parameterTypes);
		m.setAccessible(true);
		System.out.println("getDeclaredMethod=" + m.toString());
		return m;
	}

	// Public, private or protected methods of an Object
	public Method[] getDeclaredMethods(Object o) throws Exception {
		Class c = o.getClass();
		Method[] mtab = c.getDeclaredMethods();
		for (int i = 0; i < mtab.length; i++) {
			mtab[i].setAccessible(true);
			System.out.println("getDeclaredMethods mtab[" + i + "]=" + mtab[i].toString());
		}
		return mtab;
	}

	// Constructor object corresponding to public, private or protected constructor
	// with the defined parameters by the parameterTypes table
	public Constructor getDeclaredConstructor(Object o, Class... parameterTypes) throws Exception {
		Class c = o.getClass();
		Constructor cons = c.getDeclaredConstructor(parameterTypes);
		cons.setAccessible(true);
		System.out.println("getDeclaredConstructor=" + cons.toString());
		return cons;
	}

	// Public, private or protected constructors of an Object
	public Constructor[] getDeclaredConstructors(Object o) throws Exception {
		Class c = o.getClass();
		Constructor[] constab = c.getDeclaredConstructors();
		for (int i = 0; i < constab.length; i++) {
			constab[i].setAccessible(true);
			System.out.println("getDeclaredConstructors constab[" + i + "]=" + constab[i].toString());
		}
		return constab;
	}	
	
	// Modify a public, private or protected field
	public void changeDeclaredFieldValue(Object o, String name, Object val) throws Exception {
		Field f = this.getDeclaredField(o, name);
		f.setAccessible(true);
		f.set(o,val);
	}
	
	// Return a specific public, private or  protected field
	public Object getDeclaredFieldValue(Object o, String name) throws Exception {
		Field f = this.getDeclaredField(o, name);
		f.setAccessible(true);
		return f.get(o);
	}

	// Invoke a specific public, private or protected method
	public Object invokeDeclaredMethod(Object o, String name, Object[] args) throws Exception {
	  Class[] paramTypes = null;
	  if(args != null)
	  {
	    paramTypes = new Class[args.length];
	    for(int i=0;i<args.length;++i){
	      paramTypes[i] = args[i].getClass();
	    }
	  }
	  Method m = this.getDeclaredMethod(o, name, paramTypes);
	  m.setAccessible(true);
	  return m.invoke(o,args);
	}
	
	
	
	
}